class SetGol(object):
    """
    Game of Life implemented via sets

    :param board: the initial state of the board
    :type board: :py:class:`gksol.boards.PaddedBoard` or List[List[int]]

    .. describe:: gol[n]

        Return the ``n``'th row of the board as a list-like view.
    """
    def __init__(self, board):
        self.height = len(board)
        self.width = len(board[0]) if board else 0
        self.board = {(h, w) for h in range(self.height) for w in range(self.width) if board[h][w]}

    def advance(self):
        """Advance the board to the next generation"""
        next_board = set()
        for cell in self.board:
            neighbours = self._neighbour_cells(cell)
            # test if current cell survives
            if sum(nb in self.board for nb in neighbours) in {2, 3}:
                next_board.add(cell)
            # test if neighbours are born
            for nb in neighbours:
                if self._valid_cell(nb):
                    if sum(nnb in self.board for nnb in self._neighbour_cells(nb)) == 3:
                        next_board.add(nb)
        self.board = next_board

    @staticmethod
    def _neighbour_cells(cell):
        h, w = cell
        return {
            (h+i, w+j)
            for i in (-1, 0, 1)
            for j in (-1, 0, 1)
            if not i == j == 0
        }

    def _valid_cell(self, cell):
        h, w = cell
        return 0 <= h < self.height and 0 <= w < self.width

    def __getitem__(self, item):
        return GolRowView(self, item)


class GolRowView(object):
    __slots__ = ('_gol_board', '_row_index')

    def __init__(self, gol_board, row_index):
        self._gol_board = gol_board
        self._row_index = row_index

    def __getitem__(self, item):
        return (self._row_index, item) in self._gol_board.board

GOL = SetGol



import unittest
import gksol.patterns
import gksol.boards
import gksol.plain
import random


class Mixin(object):
    """Container class to shield Mixins from top level namespace where ``unittest`` would run them"""
    class GolMixin(unittest.TestCase):
        """Standard tests for the Game of Life board interface"""
        gol_cls = gksol.plain.ListGol

        @staticmethod
        def _gol_to_list(gol_board):
            return [[gol_board[line][row] for row in range(gol_board.width)] for line in range(gol_board.height)]

        def test_init_lists(self):
            """initialize with nested list [[0, 1, ...], [1, 0, ...], ...]"""
            for board in (
                [[1, 1], [1, 1]],
                [[0 for _ in range(4)], [0, 1, 1, 0], [0, 1, 1, 0], [0 for _ in range(4)]],
                [[random.randint(0, 1) for _ in range(12)]for _ in range(18)],
            ):
                with self.subTest(board=board):
                    self.assertIsInstance(self.gol_cls(board), self.gol_cls)

        def test_init_board(self):
            """initialize with gksol.boards.PaddedBoard"""
            for pattern_name in gksol.patterns.PATTERNS:
                for padding in ({}, {'top': 20}, {'left': 30}, {'top': 10, 'bottom': 12, 'left': 15, 'right': 37}):
                    board = gksol.boards.PaddedBoard(
                        getattr(gksol.patterns, pattern_name),
                        **padding
                    )
                    self.assertIsInstance(self.gol_cls(board), self.gol_cls)

        def test_getitem(self):
            """fetch fields from board by gol[line][row]"""
            for board in (
                [[1, 1], [1, 1]],
                [[0 for _ in range(4)], [0, 1, 1, 0], [0, 1, 1, 0], [0 for _ in range(4)]],
                [[random.randint(0, 1) for _ in range(12)]for _ in range(18)],
            ):
                with self.subTest(board=board):
                    instance = self.gol_cls(board)
                    self.assertEqual(
                        [[instance[line][row] for row in range(len(board[0]))] for line in range(len(board))],
                        board
                    )
                    self.assertEqual(
                        self._gol_to_list(instance),
                        board
                    )

        def test_advance(self):
            """Correct result when advancing"""
            for stable_board in (
                    [[1, 1], [1, 1]],
                    [[0, 0], [0, 0]],
                    [[0 for _ in range(4)], [0, 1, 1, 0], [0, 1, 1, 0], [0 for _ in range(4)]],
            ):
                instance_block = self.gol_cls(stable_board)
                for generation in range(10):
                    instance_block.advance()
                    self.assertEqual(self._gol_to_list(instance_block), stable_board)



import io
import itertools


def read_literal(literal, width=None, height=None):
    r"""
    Parse a GOL board from a literal to a nested list representation

    :param literal: a GOL board literal
    :param width: number of columns to enforce
    :param height: number of rows to enforce
    :return: nested list representation of the GOL board
    :rtype: list[list[bool]]

    A literal is a plain string, with rows separated by newlines. Every
    character is a field of the GOL board, and any non-empty character
    counts as alive. The use of long strings is recommended, but not required.
    The following literals produce the same board::

    .. code

        long_string = \"\"\"\\
             1
        ABC  2  DEF
             3
        \"\"\"\

        short_string = '     #     \nXXX  #  XXX\n     #     \n'

    The result is a nested list of lists, which can be indexed as ``matrix[row][column]``.
    In other words, the first index is the height and the second the width when viewed as a literal.
    Any field has a value of :py:const:`True` if it is alive, and :py:const:`False` otherwise.
    For the above literals, the following matrix is generated::

    .. code::

        [[0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0],
         [1, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1],
         [0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0]]

    The parameters ``width`` and ``height`` allow to shrink or expand the GOL board:
    Any row is trimmed or filled with :py:const:`False` to the length of ``width``.
    No more than ``height`` rows are returned, and empty rows are appended if the literal is too short.

    If any of ``width`` or ``height`` is :py:const:`None` (the default), it is computed to match
    the biggest extends of the literal.
    Note that specifying ``height`` but not ``width`` means that skipped rows are not inspected to
    derive ``width``.
    """
    if width is None:
        buffer = _read_unbounded_literal(literal, height)
        width = len(buffer[0]) if buffer else 0
    else:
        buffer = _read_bounded_literal(literal, width, height)
    if height is not None:
        if len(buffer) < height:
            buffer.extend(
                [[0] * width] * (height - len(buffer))
            )
    return buffer


def _parse_literal(literal, height):
    for line in itertools.islice(io.StringIO(literal), height):
        yield [char != ' ' for char in line if char != '\n']


def _read_bounded_literal(literal, width, height):
    buffer = []
    for line in _parse_literal(literal, height):
        line = line[:width]
        line += [0] * (width - len(line))
        buffer.append(line)
    return buffer


def _read_unbounded_literal(literal, height):
    buffer = []
    for line in _parse_literal(literal, height):
        buffer.append(line)
    if not buffer:
        return buffer
    width = max(len(line) for line in buffer)
    for line_idx in range(len(buffer)):
        buffer[line_idx].extend([0] * (width - len(buffer[line_idx])))
    return buffer



import itertools
from collections import abc as collection_abc


class PaddedBoard(object):
    def __init__(self, board, *, border=0, top=0, bottom=0, left=0, right=0):
        self.board = board
        self.top = top + border
        self.bottom = bottom + border
        self.left = left + border
        self.right = right + border
        self._board_height = len(board)
        try:
            self._board_width = len(board[0])
        except IndexError:
            self._board_width = 0

    @property
    def height(self):
        return self.top + self._board_height + self.bottom

    @property
    def width(self):
        return self.left + self._board_width + self.right

    def __getitem__(self, index):
        if index < 0:
            index += self.height
        if index < 0 or index >= self.height:
            raise IndexError('board height index out of range')
        if index < self.top or index >= self.top + self._board_height:
            return FillerRow(self.width)
        return PaddedRow(self.board[index - self.top], left=self.left, right=self.right)

    def __iter__(self):
        width = self.width
        yield from (FillerRow(width) for _ in range(self.top))
        yield from (PaddedRow(row, left=self.left, right=self.right) for row in self.board)
        yield from (FillerRow(width) for _ in range(self.bottom))

    def __len__(self):
        return self.height
    
    def __str__(self):
        return '[%s]' % ', '.join(str(line) for line in self),
        ))


class PaddedRow(collection_abc.Sequence):
    def __init__(self, row, left=0, right=0):
        self.row = row
        self.left = left
        self.right = right

    def __len__(self):
        return self.left + self.right + len(self.row)

    def __iter__(self):
        yield from (0 for _ in range(self.left))
        yield from self.row
        yield from (0 for _ in range(self.right))

    def __getitem__(self, item):
        length = len(self)
        if item < -length or item > length:
            raise IndexError('list index out of range')
        if item < 0:
            item = length - item
        if item < self.left or item >= length - self.right:
            return 0
        return self.row[item - self.left]

    def __repr__(self):
        return '%s(left=%d, right=%d, %r)' % (self.__class__.__name__, self.left, self.right, self.row)

    def __str__(self):
        return '[%s]' % ', '.join(itertools.chain(
            *('0' for _ in range(self.left)),
            [str(self.row)[1:-1]],
            *('0' for _ in range(self.right)),
        ))


class FillerRow(collection_abc.Sequence):
    def __init__(self, length):
        self._length = length

    def __len__(self):
        return self._length

    def __iter__(self):
        yield from (0 for _ in range(self._length))

    def __getitem__(self, item):
        if -self._length < item < self._length:
            return 0
        raise IndexError('list index out of range')
    
    def __repr__(self):
        return "%s(%d)" % (self.__class__.__name__, self._length)

    def __str__(self):
        return '[%s]' % (("0, " * self._length)[:-2])



from cpython.mem cimport PyMem_Malloc as malloc, PyMem_Free as free
# from libc.stdlib cimport malloc, free


cdef class CythonGol(object):
    """
    Game of Life implemented via Cython

    :param board: the initial state of the board
    :type board: :py:class:`gksol.boards.PaddedBoard` or List[List[int]]

    .. describe:: gol[n]

        Return the ``n``'th row of the board as a list-like view.
    """
    cdef int ** _board
    cdef public int height
    cdef public int width

    def __init__(self, object board):
        self.height = len(board)
        self.width = len(board[0]) if board else 0
        self._board = <int**>malloc(self.height * sizeof(int*))
        for line_idx in range(self.height):
            self._board[line_idx] = <int*>malloc(self.width * sizeof(int))
            for row_idx in range(self.width):
                self._board[line_idx][row_idx] = 1 if board[line_idx][row_idx] else 0

    cpdef advance(self):
        """Advance the board to the next generation"""
        # most of the board will be empty, so initialize to that
        cdef int ** next_board = <int**>malloc(self.height * sizeof(int*))
        for line_idx in range(self.height):
            next_board[line_idx] = <int*>malloc(self.width * sizeof(int))
            for row_idx in range(self.width):
                next_board[line_idx][row_idx] = 0
        # sieve through the board to find cells alive in the next step
        for w in range(self.width):
            for h in range(self.height):
                neighbours = self._count_neighbours(h, w)
                if neighbours == 3:
                    next_board[h][w] = 1
                elif neighbours == 2:
                    next_board[h][w] = self._board[h][w]
        for line_idx in range(self.height):
            free(self._board[line_idx])
        free(self._board)
        self._board = next_board

    cdef _count_neighbours(self, const int h, const int w):
        cdef int n_neighbours = 0
        # left and right
        if h != 0:
            n_neighbours += self._board[h - 1][w]
        if h != self.height - 1:
            n_neighbours += self._board[h + 1][w]
        # top and bottom
        if w != 0:
            n_neighbours += self._board[h][w - 1]
            if h != 0:
                n_neighbours += self._board[h - 1][w - 1]
            if h != self.height - 1:
                n_neighbours += self._board[h + 1][w - 1]
        if w != self.width - 1:
            n_neighbours += self._board[h][w + 1]
            if h != 0:
                n_neighbours += self._board[h - 1][w + 1]
            if h != self.height - 1:
                n_neighbours += self._board[h + 1][w + 1]
        return n_neighbours

    def __getitem__(self, index):
        return [self._board[index][row_index] for row_index in range(self.width)]

    def __dealloc__(self):
        for line_idx in range(self.height):
            free(self._board[line_idx])
        free(self._board)

GOL = CythonGol



class ListGol(object):
    """
    Game of Life implemented via lists

    :param board: the initial state of the board
    :type board: :py:class:`gksol.boards.PaddedBoard` or List[List[int]]

    .. describe:: gol[n]

        Return the ``n``'th row of the board as a list-like view.
    """
    def __init__(self, board):
        self._board = board
        self.height = len(board)
        self.width = len(board[0]) if board else 0

    def advance(self):
        """Advance the board to the next generation"""
        # most of the board will be empty, so efficiently initialize to that
        next_board = [[0] * self.width for _ in range(self.height)]
        for w in range(self.width):
            for h in range(self.height):
                neighbours = self._neighbours(h, w)
                if neighbours == 3:
                    next_board[h][w] = 1
                elif neighbours == 2:
                    next_board[h][w] = self._board[h][w]
        self._board = next_board

    def _neighbours(self, h, w):
        if h == 0:
            h_indizes = (0, 1)
        elif h == self.height - 1:
            h_indizes = (h - 1, h)
        else:
            h_indizes = (h - 1, h, h + 1)
        if w == 0:
            w_indizes = (0, 1)
        elif w == self.width - 1:
            w_indizes = (w - 1, w)
        else:
            w_indizes = (w - 1, w, w + 1)
        return sum(
            self._board[i][j]
            for i in h_indizes
            for j in w_indizes
            if i != h or j != w
        )

    def __getitem__(self, item):
        return self._board[item]

GOL = ListGol



import sys
import argparse
import time

from . import patterns
from . import reader
from . import boards
from . import render

try:
    import pyximport
except ImportError:
    pass
else:
    pyximport.install()


def board_help():
    max_pattern_len = max(len(pat) for pat in patterns.PATTERNS)
    print('Specify either an existing pattern or a new board', file=sys.stderr)
    print('Patterns:', file=sys.stderr)
    for index in range(0, len(patterns.PATTERNS), 3):
        print(
            '\t', '  '.join(elem.ljust(max_pattern_len) for elem in patterns.PATTERNS[index:index + 3]),
            file=sys.stderr
        )


def load_board(board_name, *padding):
    if not board_name:
        board_help()
        raise ValueError
    cli_arg_map = {kwarg[0]: kwarg for kwarg in ('top', 'bottom', 'left', 'right')}
    kwargs = {}
    for pad in padding:
        if pad[0] not in cli_arg_map:
            val = float(pad) if pad == 'inf' else int(pad)
            kwargs = {kwarg: val for kwarg in cli_arg_map.values()}
        else:
            direction, amount = pad[0], pad[1:]
            kwargs[cli_arg_map[direction]] = float(amount) if amount == 'inf' else int(amount)
    board_literal = getattr(patterns, board_name, board_name)
    raw_board = reader.read_literal(board_literal)
    return boards.PaddedBoard(raw_board, **kwargs)


def load_gol(module_name):
    __import__(module_name)
    return sys.modules[module_name].GOL


def main(options):
    board = load_board(options.board, *options.padding)
    gol_class = load_gol(options.gol_class)
    gol = gol_class(board)
    display = getattr(render, options.renderer)()
    display.show(gol)
    start_time = time.time()
    _delay, _generations, _every = options.delay, options.generations, options.every
    for generation in range(_generations):
        time.sleep(_delay)
        gol.advance()
        if generation % _every == 0:
            display.show(gol, title='%d/%d %6.3f' % (generation, _generations, (100 / (time.time() - start_time))))
        if generation % 100 == 99:
            print('%10d' % generation, '%6.3f' % (100 / (time.time() - start_time)), 'FPS')
            start_time = time.time()


CLI = argparse.ArgumentParser('')
CLI.add_argument(
    '-c', '--class',
    dest='gol_class',
    help='GOL implementation',
    default='gksol.sparse',
)
CLI_BOARD = CLI.add_argument_group('Board settings')
CLI_BOARD.add_argument(
    '-b', '--board',
    help='Initial board pattern [%(default)s]',
    default='PENTOMINO',
)
CLI_BOARD.add_argument(
    '-p', '--padding',
    help='Padding for all sides as "23" or [tblr] direction as "r23"',
    nargs='*',
    default=['20', 'l40', 'r40'],
)
CLI_SIM = CLI.add_argument_group('Simulation settings')
CLI_SIM.add_argument(
    '-g', '--generations',
    help='Generations to simulate',
    type=int,
    default=100,
)
CLI_SIM.add_argument(
    '-d', '--delay',
    help='Delay between generations',
    type=float,
    default=0.0,
)
CLI_SHOW = CLI.add_argument_group('Display settings')
CLI_SHOW.add_argument(
    '-r', '--renderer',
    help='renderer to display the board',
    choices=render.__all__,
    default='TextDisplay',
)
CLI_SHOW.add_argument(
    '-e', '--every',
    help="Show only every e'th frame",
    default=1,
    type=int,
)

main(CLI.parse_args())



import sys
import cpy2py
import platform
import atexit


class NullDisplay(object):
    """
    Dummy renderer for a Game of Life board or nested list

    This renderer does nothing.
    """
    def __init__(self, *args):
        pass

    def show(self, gol, title='<Title>'):
        pass


class TextDisplay(object):
    """
    Renderer for a Game of Life board or nested list for text terminals

    :param height: the maximum height to display the board
    :type height: int
    :param width: the maximum width to display the board
    :type width: int

    This renderer requires ANSII support by the terminal.
    """
    def __init__(self, height=128, width=128):
        self.height = height
        self.width = width

    def show(self, gol, title='<Title>'):
        self._show_array(gol, title=title)

    def _show_array(self, gol_array, title):
        sys.stdout.write("\x1b[H")
        draw_width = min(self.width, gol_array.width)
        print('-', title, '-' * max(draw_width - 3 - len(title), 0))
        for h in range(min(self.height, gol_array.height)):
            row = gol_array[h]
            print(''.join('#' if row[w] else ' ' for w in range(min(self.width, gol_array.width))))
        print('-' * gol_array.width)


class NativeMPLDisplay(object):
    """
    Renderer for a Game of Life board or nested list for text terminals

    :param height: the maximum height to display the board
    :type height: int
    :param width: the maximum width to display the board
    :type width: int

    This renderer requires :py:mod:`matplotlib`.
    """
    def __init__(self, height=640, width=640):
        from matplotlib import pyplot
        self.height = height
        self.width = width
        pyplot.draw()

    def show(self, gol, title='<Title>'):
        self._show_mpl(gol, title=title)

    def _show_mpl(self, gol_array, title):
        from matplotlib import pyplot
        draw_height = min(self.height, gol_array.height)
        draw_width = min(self.width, gol_array.width)
        content = []
        for line_idx in range(draw_height):
            line = gol_array[line_idx]
            content.append([line[row_idx] for row_idx in range(draw_width)])
        pyplot.clf()
        pyplot.title(title)
        pyplot.imshow(content)
        pyplot.pause(0.000001)


class CPyMPLDisplay(cpy2py.TwinObject):
    """
    Renderer for a Game of Life board or nested list for text terminals

    :param height: the maximum height to display the board
    :type height: int
    :param width: the maximum width to display the board
    :type width: int

    This renderer requires :py:mod:`matplotlib` and :py:mod:`cpy2py`.
    """
    __twin_id__ = 'python3'

    def __init__(self, height=640, width=640):
        from matplotlib import pyplot
        self.height = height
        self.width = width
        pyplot.draw()

    @cpy2py.localmethod
    def show(self, gol, title='<Title>'):
        draw_height = min(self.height, gol.height)
        draw_width = min(self.width, gol.width)
        content = []
        for line_idx in range(draw_height):
            line = gol[line_idx]
            content.append([line[row_idx] for row_idx in range(draw_width)])
        self._show_mpl(content, title=title)

    def _show_mpl(self, gol_array, title):
        from matplotlib import pyplot
        pyplot.clf()
        pyplot.title(title)
        pyplot.imshow(gol_array)
        pyplot.pause(0.000001)

if platform.python_implementation() == 'PyPy':
    from cpy2py.ipyc import ipyc_socket
    draw_twinterpreter = cpy2py.TwinMaster(CPyMPLDisplay.__twin_id__, ipyc=ipyc_socket.DuplexSocketIPyC)
    draw_twinterpreter.start()
    atexit.register(draw_twinterpreter.stop)
    MPLDisplay = CPyMPLDisplay
else:
    MPLDisplay = NativeMPLDisplay

__all__ = ['TextDisplay', 'MPLDisplay', 'NullDisplay']



#!/usr/bin/env python
import os
import sys
import platform
from setuptools import setup, find_packages

repo_base_dir = os.path.abspath(os.path.dirname(__file__))
# pull in the packages metadata
package_about = {}
with open(os.path.join(repo_base_dir, "gksol", "__about__.py")) as about_file:
    exec(about_file.read(), package_about)

with open(os.path.join(repo_base_dir, 'README.rst'), 'r') as README:
    long_description = README.read()

cmdclass = {}
extensions = []

# CPython/PyPy
dependencies = ['cpy2py']
if platform.python_implementation() == 'CPython':
    dependencies.append('cython')
    dependencies.append('matplotlib')
    import Cython.Distutils
    from distutils.extension import Extension
    for file_path in (os.path.join('gksol', 'compiled.pyx'),):
        module_path = os.path.splitext(file_path)[0].replace(os.sep, '.')
        for compiled_file in (os.path.splitext(file_path)[0] + ext for ext in ('.so', '.c')):
            if os.path.isfile(compiled_file):
                os.unlink(compiled_file)
        extensions.append(Extension(name=module_path, sources=[file_path]))
    if extensions:
        cmdclass = {'build_ext': Cython.Distutils.build_ext}


if __name__ == '__main__':
    setup(
        name=package_about['__title__'],
        version=package_about['__version__'],
        description=package_about['__summary__'],
        long_description=long_description.strip(),
        author=package_about['__author__'],
        author_email=package_about['__email__'],
        url=package_about['__url__'],
        packages=find_packages(),
        ext_modules=extensions,
        cmdclass=cmdclass,
        # dependencies
        install_requires=dependencies,
        # metadata for package search
        license='MIT',
        # https://pypi.python.org/pypi?%3Aaction=list_classifiers
        classifiers=[
            'Development Status :: 4 - Beta',
            'License :: OSI Approved :: MIT License',
            'Intended Audience :: Education',
            'Topic :: Education',
            'Programming Language :: Python :: 3 :: Only',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: Implementation :: CPython',
            'Programming Language :: Python :: Implementation :: PyPy'
        ],
        keywords=package_about['__keywords__'],
        # unit tests
        test_suite='gksol.gksol_unittests',
    )



__title__ = 'gksol'
__summary__ = 'Game of Life Tutorial Implementation and Framework'
__url__ = 'https://github.com/MaineKuehn/gksol'

__version__ = '0.2.0'
__author__ = 'Eileen Kuehn, Max Fischer'
__email__ = 'maxfischer2781@gmail.com'
__copyright__ = '2017 %s' % __author__
__keywords__ = 'gridkaschool tutorial game-of-life gol'
